/**********************************************************************************/
/*    Demo program for:								  */
/*	  Board: MSP430-P2274_Rev.A						  */
/*    Manufacture: OLIMEX                                                   	  */
/*	  COPYRIGHT (C) 2010							  */
/*    Designed by: Engineer Penko T. Bozhkov                                      */
/*    Module Name    :  main module                                               */
/*    File   Name    :  main.c                                                    */
/*    Revision       :  initial                                                   */
/*    Date           :  16.03.2010                                                */
/*    Built with IAR Embedded Workbench Version: 4.21                             */
/**********************************************************************************/
#include  <intrinsics.h>
#include  <io430x22x4.h>

#define _BIC_SR(reg) __bic_SR_register(reg)
#define _BIS_SR(reg) __bis_SR_register(reg)
#define Clock_Ok      0x00
#define Clock_Fail    0x01
#define LFXT1_LF_FAIL           Test_Flags_Reg |= 0x01;
#define LFXT1_LF_OK             Test_Flags_Reg &= ~0x01;
#define LFXT1_LF_Check          (Test_Flags_Reg & 0x01)
#define LFXT1_HF_FAIL           Test_Flags_Reg |= 0x02;
#define LFXT1_HF_OK             Test_Flags_Reg &= ~0x02;
#define LFXT1_HF_Check          (Test_Flags_Reg & 0x02)

#define LED1_ON                 P3OUT |= 0x40; //P3DIR |= 0x40;
#define LED1_OFF                P3OUT &= ~0x40; //P3DIR |= 0x40;
#define LED1_Check              (P3IN & 0x40)
#define LED2_ON                 P3OUT |= 0x80; //P3DIR |= 0x80;
#define LED2_OFF                P3OUT &= ~0x80; //P3DIR |= 0x80;


unsigned char   LFXT1_OSC_Counter;
unsigned char	Test_Flags_Reg;

/**********************************************************************************/
/*  Function name: Delay                                                          */
/*  	Parameters                                                                */
/*          Input   :  No	                                                  */
/*          Output  :  No	                                                  */
/*	Action: Simple delay							  */
/**********************************************************************************/
void Delay(volatile unsigned int delay_counter){
	while(delay_counter){delay_counter--;}
}


/**********************************************************************************/
/*  Function name: Ports_initial_initialization                                   */
/*  	Parameters                                                                */
/*          Input   :  No	                                                  */
/*          Output  :  No	                                                  */
/*	Action: Define initial ports state and directions.			  */
/**********************************************************************************/
void Ports_initial_initialization(void)
{
   //input_data=P1IN;          //Read only register
  P1OUT = 0x00;               // The outputs are low
  P1SEL = 0x00;               // Peripheral module function: I/O function is selected.
  P1DIR = 0x00;               // 1 -> Output; 0 -> Input; All inputs (But1 and But2 also) 
  P1REN = 0x00;               // Pullup/pulldown resistor disabled
  P1IES = 0x00;               // Interrupt Edge Select Registers
  P1IE = 0x00;                // Disable all PORTx interrupts
  P1IFG = 0x00;               // Clear all interrupt flags

   //input_data=P2IN;          //Read only register
  P2OUT = 0x00;               // The outputs are low
  P2SEL = 0xC0;               // Peripheral module function: Enabled for P2.6(XIN) and P2.7(XOUT), all other I/O function is selected.
  P2DIR = 0x00;               // 1 -> Output; 0 -> Input
  P2REN = 0x00;               // Pullup/pulldown resistor disabled
  P2IES = 0x00;               // Interrupt Edge Select Registers
  P2IE = 0x00;                // Disable all PORTx interrupts
  P2IFG = 0x00;               // Clear all interrupt flags

   //input_data=P3IN;          //Read only register
  P3OUT = 0x00;               // The outputs are low
  P3SEL = 0x30;               // Peripheral module function: enabled for UCA0TXD and UCA0RXD
  P3DIR = 0xD0;               // 1 -> Output; 0 -> Input; UCA0TXD output, LED1 and LED2 outputs
  P3REN = 0x00;               // Pullup/pulldown resistor disabled

  //input_data=P4IN;          //Read only register
  P4OUT = 0x00;               // The outputs are low
  P4SEL = 0x00;               // Peripheral module function: I/O function is selected.
  P4DIR = 0x00;               // 1 -> Output; 0 -> Input;
  P4REN = 0x00;               // Pullup/pulldown resistor disabled
}


/**********************************************************************************/
/*  Function name: Set_System_Clock                                               */
/*  	Parameters                                                                */
/*          Input   :  No	                                                  */
/*          Output  :  No	                                                  */
/*	Action: Set CPU clock source.                   			  */
/**********************************************************************************/
char Set_System_Clock(void)
{
  unsigned char System_Clock_State = 0;
  unsigned char Crystal_counter = 0;
  
  // Switch MCLK from DCO to LFXT1 crystal oscillator in LF mode:
  _BIC_SR(OSCOFF);              // 1. Enable the LFXT1 crystal oscillator
  BCSCTL1 = 0x07;               // 2. Select LFXT1 in Low frequency mode
  BCSCTL3 = 0x0C;               // Change Oscillator capacitor: 01 ~12.5 pF, Set -> LFXT1 range: 32768Hz 
  Crystal_counter = 0x00;
  do{
    IFG1 &= ~0x02;              // 3. Clear only OFIFG!!!
    Delay(200);                 // 4. Wait more than 50us
    Crystal_counter++;
    if(Crystal_counter == 100){ BCSCTL3 = 0x08; }           // Change Oscillator capacitor: 01 ~10 pF
    else if(Crystal_counter == 150){ BCSCTL3 = 0x04; }     // Change Oscillator capacitor: 01 ~6 pF
    else if(Crystal_counter == 200){ BCSCTL3 = 0x00; }     // Change Oscillator capacitor: 00 ~1 pF
    else if(Crystal_counter == 250){                       // LFXT1 oscillator fault
      LFXT1_LF_FAIL;
      System_Clock_State = Clock_Fail;
      break;
    }         
  }while(IFG1 & 0x02);
  if(!LFXT1_LF_Check){
    // LFXT1 crystal oscillator is selected!
    BCSCTL2 = 0xC8;               // 5. Select LFXT1CLK -> MCLK: LFXT1CLK, Divider for MCLK: 00, Select SMCLK: LFXT1CLK, Divider for SMCLK: 00
    System_Clock_State = Clock_Ok;
    LFXT1_LF_OK;
    goto  Exit_System_Clock_Set;  // System clock is set to 32768Hz Qrystal oscillator, so EXIT
  }
  
  // Switch MCLK from DCO to LFXT1 crystal oscillator in HF mode if LF mode fail:
  _BIC_SR(OSCOFF);              // 1. Enable the LFXT1 crystal oscillator
  BCSCTL1 |= 0x40;              // 2. Select LFXT1 in High frequency mode, XTS = 1
  BCSCTL3 = 0x00;               //    Set -> LFXT1 frequency range: 0.4 - 1-MHz crystal or resonator
  Crystal_counter = 0x00;
  do{
    IFG1 &= ~0x02;              // 3. Clear only OFIFG!!!
    Delay(200);                 // 4. Wait more than 50us
    Crystal_counter++;
    if(Crystal_counter == 100){ BCSCTL3 = 0x10; }           // LFXT1 frequency range: 1 - 3-MHz crystal or resonator
    else if(Crystal_counter == 150){ BCSCTL3 = 0x20; }     // LFXT1 frequency range: 3 - 16-MHz crystal or resonator
    else if(Crystal_counter == 200){ BCSCTL3 = 0x30; }     // LFXT1 frequency range: Digital external 0.4 - 16-MHz clock source
    else if(Crystal_counter == 250){                       // LFXT1 oscillator fault
      LFXT1_HF_FAIL;
      System_Clock_State = Clock_Fail;
      break;
    }         
  }while(IFG1 & 0x02);
  if(!LFXT1_HF_Check){
    // LFXT1 crystal oscillator is selected!
    BCSCTL2 = 0xC8;               // 5. Select LFXT1CLK -> MCLK: LFXT1CLK, Divider for MCLK: 00, Select SMCLK: LFXT1CLK, Divider for SMCLK: 00
    System_Clock_State = Clock_Ok;
    LFXT1_HF_OK;
    goto  Exit_System_Clock_Set;  // System clock is set, so EXIT
  }
  
Exit_System_Clock_Set:
  if(System_Clock_State == Clock_Ok){
    IFG1 &= ~0x1F;              // Clear WDTIFG, RSTIFG, PORIFG, OFIFG and NMIIFG
    _BIS_SR(SCG1);              // turn off the SMCLK
    _BIS_SR(SCG0);              // turn off the DCO dc generator
  }
  return  System_Clock_State;
}


/**********************************************************************************/
/*  Function name: init_devices	                                                  */
/*  	Parameters                                                                */
/*          Input   :  No			     		                  */
/*          Output  :  No                                      	            	  */
/*	Action: Initialize all used MSP430F2274 peripheral devices.	  	  */
/**********************************************************************************/
void init_devices(void){
  _BIC_SR(GIE); // Disable interrupts during initialization process
  
  /********** 1.Ports initialization ***********/
  Ports_initial_initialization();
  /********** 2.Select System Clock  ***********/
  if(Set_System_Clock() != Clock_Ok){
    LED1_ON;    // If System clock is Fail, CPU is clocked by 1.1MHz DCO Clock
    while(1){}  // RTC clock FAIL!
  }
  else{
    LED2_ON; // If System clock is Set, CPU is clocked by selected external Clock
  }
  /********** 3.Peripherals initialization ***********/
  // 3.1. Timer_A
  // =============
  TACTL = 0x0004;   // Timer_A clear.
  TACCTL0 = 0x0010; // Timer_A Capture/compare interrupt enable.
  TACCR0 = 0x007F;  // Set TACCR0 value
  TACTL = 0x0110;   // Selected: ACLK, No divider, Up mode.
  //TAIV = 0x000A;

  _BIS_SR(GIE); // Global Interrupt enabled. Do this at the END of the initialization!!!!!!!!
}


/**********************************************************************************/
/*  Function name: main	                                                  	  */
/*  	Parameters                                                                */
/*          Input   :  No			     		                  */
/*          Output  :  No                                      	            	  */
/*	Action: Call "init_devices".                    	  		  */
/**********************************************************************************/
void  main(void){
  WDTCTL = 0x5A80;              // Stop WDT
  init_devices();
  while(1){
    
  }
}


/**********************************************************************************/
/*  Function name: TIMERA_Capture_Compare_ISR                                  	  */
/*  	Parameters                                                                */
/*          Input   :  No			     		                  */
/*          Output  :  No                                      	            	  */
/*	Action:  Toggle LED1 and LED2 with frequency 1Hz.                         */
/**********************************************************************************/
#pragma vector=TIMERA0_VECTOR
__interrupt void TIMERA_Capture_Compare_ISR(void){
  // Toggle LED1 & LED2
  LFXT1_OSC_Counter++;
  if(LFXT1_OSC_Counter == 128){
    LFXT1_OSC_Counter = 0;
    if(LED1_Check){
      LED1_OFF;
      LED2_ON;
    }
    else{
      LED1_ON;
      LED2_OFF;
    }
  }
}




